@@ -2,14 +2,12 @@
 """
 import re
 from functools import partial
-from dataclasses import dataclass
 
 from ovs_dbg.flow import Flow, Section
 
 from ovs_dbg.kv import (
     KVParser,
     KVDecoders,
-    ParseError,
     nested_kv_decoder,
     decode_nested_kv,
 )
@@ -27,20 +25,38 @@ from ovs_dbg.decoders import (
     EthMask,
     decode_free_output,
     decode_flag,
-    decode_ip_port_range,
     decode_nat,
 )
 
 
 class ODPFlow(Flow):
+    def __init__(self, sections, raw="", id_=None):
+        """Constructor"""
+        super(ODPFlow, self).__init__(sections, raw, id_)
+
+    def __str__(self):
+        if self._orig:
+            return self._orig
+        else:
+            return self.to_string()
+
+    def to_string(self):
+        """Print a text representation of the flow"""
+        string = "Info: {}\n" + self.info
+        string += "Match : {}\n" + self.match
+        string += "Actions: {}\n " + self.actions
+        return string
+
+
+class ODPFlowFactory:
     """Datapath Flow"""
 
-    def __init__(self, sections, raw="", id=None):
-        """Constructor"""
-        super(ODPFlow, self).__init__(sections, raw, id)
+    def __init__(self):
+        self.info_decoders = self._info_decoders()
+        self.match_decoders = self._match_decoders()
+        self.action_decoders = self._action_decoders()
 
-    @classmethod
-    def from_string(cls, odp_string, id=None):
+    def from_string(self, odp_string, id=None):
         """Parse a odp flow string
 
         The string is expected to have the follwoing format:
@@ -58,16 +74,20 @@ class ODPFlow(Flow):
         # If UFID present, parse it and
         ufid_pos = odp_string.find("ufid:")
         if ufid_pos >= 0:
-            ufid_string = odp_string[ufid_pos : (odp_string[ufid_pos:].find(",") + 1)]
+            ufid_string = odp_string[
+                ufid_pos : (odp_string[ufid_pos:].find(",") + 1)
+            ]
             ufid_parser = KVParser(KVDecoders({"ufid": decode_default}))
             ufid_parser.parse(ufid_string)
             if len(ufid_parser.kv()) != 1:
-                raise ValueError("malformed odp flow: {}", odp_string)
-            sections.append(Section("ufid", ufid_pos, ufid_string, ufid_parser.kv()))
+                raise ValueError("malformed odp flow: %s" % odp_string)
+            sections.append(
+                Section("ufid", ufid_pos, ufid_string, ufid_parser.kv())
+            )
 
         action_pos = odp_string.find("actions:")
         if action_pos < 0:
-            raise ValueError("malformed odp flow: {}", odp_string)
+            raise ValueError("malformed odp flow: %s" % odp_string)
 
         # rest of the string is between ufid and actions
         rest = odp_string[
@@ -80,27 +100,34 @@ class ODPFlow(Flow):
         field_parts = rest.lstrip(" ").partition(" ")
 
         if len(field_parts) != 3:
-            raise ValueError("malformed odp flow: {}", odp_string)
+            raise ValueError("malformed odp flow: %s" % odp_string)
 
         match = field_parts[0]
         info = field_parts[2]
 
-        info_decoders = cls._info_decoders()
-        iparser = KVParser(KVDecoders(info_decoders))
+        iparser = KVParser(KVDecoders(self.info_decoders))
         iparser.parse(info)
         isection = Section(
-            name="info", pos=odp_string.find(info), string=info, data=iparser.kv()
+            name="info",
+            pos=odp_string.find(info),
+            string=info,
+            data=iparser.kv(),
         )
         sections.append(isection)
 
-        mparser = cls._match_parser()
+        mparser = KVParser(KVDecoders(self.match_decoders))
         mparser.parse(match)
         msection = Section(
-            name="match", pos=odp_string.find(match), string=match, data=mparser.kv()
+            name="match",
+            pos=odp_string.find(match),
+            string=match,
+            data=mparser.kv(),
         )
         sections.append(msection)
 
-        aparser = cls._action_parser()
+        aparser = KVParser(
+            KVDecoders(self.action_decoders, default_free=decode_free_output)
+        )
         aparser.parse(actions)
         asection = Section(
             name="actions",
@@ -111,10 +138,10 @@ class ODPFlow(Flow):
         )
         sections.append(asection)
 
-        return cls(sections, odp_string, id)
+        return ODPFlow(sections, odp_string, id)
 
     @classmethod
-    def _action_parser(cls):
+    def _action_decoders(cls):
         _decoders = {
             "drop": decode_flag,
             "lb_output": decode_int,
@@ -169,7 +196,7 @@ class ODPFlow(Flow):
                         ),
                         "userdata": decode_default,
                         "actions": decode_flag,
-                        "tunnel_out_port": decode_int,
+                        "tunnel_out_port": decode_default,
                         "push_eth": nested_kv_decoder(
                             KVDecoders(
                                 {
@@ -236,47 +263,41 @@ class ODPFlow(Flow):
             KVDecoders(decoders=_decoders, default_free=decode_free_output)
         )
 
-        return KVParser(
-            KVDecoders(
-                decoders={
-                    **_decoders,
-                    # "clone": nested_kv_decoder(KVDecoders(_decoders)),
-                    "sample": nested_kv_decoder(
-                        KVDecoders(
-                            {
-                                "sample": (lambda x: float(x.strip("%"))),
-                                "actions": nested_kv_decoder(
-                                    KVDecoders(
-                                        decoders=_decoders,
-                                        default_free=decode_free_output,
-                                    )
-                                ),
-                            }
-                        )
-                    ),
-                    "check_pkt_len": nested_kv_decoder(
-                        KVDecoders(
-                            {
-                                "size": decode_int,
-                                "gt": nested_kv_decoder(
-                                    KVDecoders(
-                                        decoders=_decoders,
-                                        default_free=decode_free_output,
-                                    )
-                                ),
-                                "le": nested_kv_decoder(
-                                    KVDecoders(
-                                        decoders=_decoders,
-                                        default_free=decode_free_output,
-                                    )
-                                ),
-                            }
-                        )
-                    ),
-                },
-                default_free=decode_free_output,
-            )
-        )
+        return {
+            **_decoders,
+            "sample": nested_kv_decoder(
+                KVDecoders(
+                    {
+                        "sample": (lambda x: float(x.strip("%"))),
+                        "actions": nested_kv_decoder(
+                            KVDecoders(
+                                decoders=_decoders,
+                                default_free=decode_free_output,
+                            )
+                        ),
+                    }
+                )
+            ),
+            "check_pkt_len": nested_kv_decoder(
+                KVDecoders(
+                    {
+                        "size": decode_int,
+                        "gt": nested_kv_decoder(
+                            KVDecoders(
+                                decoders=_decoders,
+                                default_free=decode_free_output,
+                            )
+                        ),
+                        "le": nested_kv_decoder(
+                            KVDecoders(
+                                decoders=_decoders,
+                                default_free=decode_free_output,
+                            )
+                        ),
+                    }
+                )
+            ),
+        }
 
     @classmethod
     def _tnl_action_decoder(cls):
@@ -394,15 +415,11 @@ class ODPFlow(Flow):
         }
 
     @classmethod
-    def _match_parser(cls):
-        return KVParser(
-            KVDecoders(
-                {
-                    **cls._field_decoders(),
-                    "encap": nested_kv_decoder(KVDecoders(cls._field_decoders())),
-                }
-            )
-        )
+    def _match_decoders(cls):
+        return {
+            **cls._field_decoders(),
+            "encap": nested_kv_decoder(KVDecoders(cls._field_decoders())),
+        }
 
     @classmethod
     def _field_decoders(cls):
@@ -452,11 +469,11 @@ class ODPFlow(Flow):
                         "erspan": nested_kv_decoder(
                             KVDecoders(
                                 {
-                                    "ver": decode_int,
-                                    "idx": decode_int,
+                                    "ver": Mask8,
+                                    "idx": Mask32,
                                     "sid": decode_int,
-                                    "dir": decode_int,
-                                    "hwid": decode_int,
+                                    "dir": Mask8,
+                                    "hwid": Mask8,
                                 }
                             )
                         ),
@@ -466,8 +483,8 @@ class ODPFlow(Flow):
                                     "gbp": nested_kv_decoder(
                                         KVDecoders(
                                             {
-                                                "id": decode_int,
-                                                "flags": decode_int,
+                                                "id": Mask16,
+                                                "flags": Mask8,
                                             }
                                         )
                                     )
@@ -638,8 +655,8 @@ class ODPFlow(Flow):
 
 def decode_geneve(mask, value):
     """
-    Decode geneve options. Used for both tnl_push(header(geneve(options()))) action
-    and tunnel(geneve()) match.
+    Decode geneve options. Used for both tnl_push(header(geneve(options())))
+    action and tunnel(geneve()) match.
 
     It has the following format:
 
