@@ -10,10 +10,26 @@ from rich.console import Console
 from rich.text import Text
 from rich.style import Style
 from rich.color import Color
+from rich.panel import Panel
+from rich.emoji import Emoji
 
 from ovs_dbg.ofparse.format import FlowFormatter, FlowBuffer, FlowStyle
 
 
+def file_header(name):
+    return Panel(
+        Text(
+            Emoji.replace(":scroll:")
+            + " "
+            + name
+            + " "
+            + Emoji.replace(":scroll:"),
+            style="bold",
+            justify="center",
+        )
+    )
+
+
 class ConsoleBuffer(FlowBuffer):
     """ConsoleBuffer implements FlowBuffer to provide console-based text
     formatting based on rich.Text
@@ -88,7 +104,11 @@ class ConsoleFormatter(FlowFormatter):
     def __init__(self, opts=None, console=None, **kwargs):
         super(ConsoleFormatter, self).__init__()
         style = self.style_from_opts(opts)
-        self.console = console or Console(no_color=(style is None), **kwargs)
+        self.console = console or Console(
+            no_color=(style is None),
+            color_system=(None if not style else "256"),
+            **kwargs
+        )
         self.style = style or FlowStyle()
 
     def style_from_opts(self, opts):
@@ -131,7 +151,8 @@ def hash_pallete(hue, saturation, value):
     HSV_tuples = itertools.product(hue, saturation, value)
     RGB_tuples = map(lambda x: colorsys.hsv_to_rgb(*x), HSV_tuples)
     styles = [
-        Style(color=Color.from_rgb(r * 255, g * 255, b * 255)) for r, g, b in RGB_tuples
+        Style(color=Color.from_rgb(r * 255, g * 255, b * 255))
+        for r, g, b in RGB_tuples
     ]
 
     def get_style(string):
@@ -141,6 +162,29 @@ def hash_pallete(hue, saturation, value):
     return get_style
 
 
+def heat_pallete(min_value, max_value):
+    """Generates a color pallete based on the 5-color heat pallete so that
+    for each value between min and max a color is returned that represents it's
+    relative size
+    Args:
+        min_value (int): minimum value
+        max_value (int) maximum value
+    """
+    h_min = 0  # red
+    h_max = 220 / 360  # blue
+
+    def heat(value):
+        if max_value == min_value:
+            r, g, b = colorsys.hsv_to_rgb(h_max / 2, 1.0, 1.0)
+        else:
+            normalized = (int(value) - min_value) / (max_value - min_value)
+            hue = ((1 - normalized) + h_min) * (h_max - h_min)
+            r, g, b = colorsys.hsv_to_rgb(hue, 1.0, 1.0)
+        return Style(color=Color.from_rgb(r * 255, g * 255, b * 255))
+
+    return heat
+
+
 def print_context(console, opts):
     """
     Returns a printing context
